//
//  scViewController.m
//  sc
//
//  Created by xcc on 09/08/15.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//

#import "scViewController.h"
#import "testView.h"

/*
	UIScrollViewのテスト用クラス
	呼び出されたメソッドをコンソール出力する。
*/
@interface testUIScrollView : UIScrollView {
}
@end
@implementation testUIScrollView
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	printf("testUIScrollView touchesBegan\n");
	[super touchesBegan:touches withEvent:event];
}
- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	printf("testUIScrollView touchesEnded\n");
	[super touchesEnded:touches withEvent:event];
}
- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	printf("testUIScrollView touchesMoved\n");
	[super touchesMoved:touches withEvent:event];
}
/*
	自分が持っているUIViewのリストアップ
*/
static void printviews(UIView* inView, int indent)
{
	CGRect r = inView.frame;
	for (int i = 0; i < indent; i++) printf(" ");
	printf("view = %s  (%.1lf,%.1lf,%.1lf,%.1lf)\n", 
		[NSStringFromClass([inView class]) UTF8String], 
		r.origin.x, r.origin.y, r.size.width, r.size.height);
	indent++;
	NSArray* viewlist = [inView subviews];
	for (UIView* view in viewlist) {
		printviews(view, indent);
	}
}
- (UIView *)hitTest:(CGPoint)point withEvent:(UIEvent *)event {
	printf("testUIScrollView hitTest\n  view list up\n");
	printviews(self, 4);
	UIView* view = [super hitTest:point withEvent:event];
	printf("  return view = %s\n", [NSStringFromClass([view class]) UTF8String]);
	return view;
}

#if __IPHONE_OS_VERSION_MIN_REQUIRED <= __IPHONE_2_2		//	2.2.1では以下の2つのメソッドを自前で準備する必要が有る
-(void)zoomToRect:(CGRect)r animated:(BOOL)animated {
}

-(void)setZoomScale:(float)scale animated:(BOOL)animated {
}
#endif
@end


@implementation scViewController



/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
	[super loadView];

	scrollview = [[testUIScrollView alloc] initWithFrame:self.view.bounds];
	scrollview.delaysContentTouches = NO;	//	フリック時もタッチイベントを送るようにするならNO。
	[self.view addSubview:scrollview];
	testview = [[testView alloc] initWithFrame:CGRectMake(0, 0, 500, 500)];
	[testview setDelegate:self];
	[scrollview addSubview:testview];
	[scrollview setContentSize:testview.frame.size];	//	スクロール範囲設定
	[scrollview setDelegate:self];			//	デリゲート設定
	[scrollview setMinimumZoomScale:0.5];	//	ズームアウト下限設定
	[scrollview setMaximumZoomScale:2.0];	//	ズームイン上限設定。
	scrollview.bounces = NO;				//	NOだとドラッグで範囲外までスクロールしてから戻るという動作を禁止する。
	scrollview.bouncesZoom = YES;			//	YESだとピンチで許可したズーム範囲外までズームを許してから範囲内に戻るという動作をゆるす。
}


/*
// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
}
*/


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

/*
	この移譲メソッドでズーム対象のUIViewを返す事で、UIScrollViewはピンチジェスチャーに対応してくれる。
	ただし、setMinimumZoomScaleかsetMaximumZoomScaleで1.0以外を指定していないと意味が無い。
*/
- (UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView {
    UIView *view = nil;
    if (scrollView == scrollview) {
        view = testview;
    }
    return view;
}

- (void)scrollViewDidEndZooming:(UIScrollView *)inScrollView withView:(UIView *)view atScale:(float)scale {
}

/*
	シングルタップ対応
	tapLocationを中心にズームアウトする。
*/
-(void)view:(testView*)view gotSingleTapAtPoint:(CGPoint)tapLocation {
    CGRect r;
	r.size.height = scrollview.frame.size.height * 0.5;
    r.size.width  = scrollview.frame.size.width  * 0.5;
	r.origin.x = tapLocation.x - (r.size.width  / 2.0);
    r.origin.y = tapLocation.y - (r.size.height / 2.0);
    [scrollview zoomToRect:r animated:YES];
}

/*
	シングルタップ対応
	tapLocationを中心に1：1に戻す。
*/
-(void)view:(testView*)view gotDoubleTapAtPoint:(CGPoint)tapLocation {
    CGRect r;
	r.size.height = scrollview.frame.size.height;
    r.size.width  = scrollview.frame.size.width;
	r.origin.x = tapLocation.x - (r.size.width  / 2.0);
    r.origin.y = tapLocation.y - (r.size.height / 2.0);
    [scrollview zoomToRect:r animated:YES];
}


@end
